def valid_base_dn(base_dn: str) -> None:
    for att in ['o', 'dc', 'ou']:
        if base_dn.startswith(att + '='):
            break
    else:
        raise ValueError('La racine doit débuter par une organisation (o=), une composante du domaine (dc=) ou une unité organisationnelle (ou=)')


def calc_ldapclient_base_dn(ldap_base_dn: str,
                            family_name: str=None,
                            base: bool=False,
                            group: bool=False,
                            ) -> str:
    if ldap_base_dn is None:
        return
    if family_name == 'all':
        family_name = None
        base = True
    if group:
        return f'ou=groups,{ldap_base_dn}'
    if not ldap_base_dn.startswith('ou=accounts,'):
        base_name = f'ou=accounts,{ldap_base_dn}'
    else:
        base_name = ldap_base_dn
    if base:
        return base_name
    if not family_name:
        return f'ou=users,{base_name}'
    base_name = f'ou=families,{base_name}'
    if family_name != '-':
        base_name = f'ou={family_name},{base_name}'
    return base_name


class _Undefined:
    pass


_undefined = _Undefined()


def get_default_base_dn(server_name: str) -> str:
    if not server_name or '.' not in server_name:
        return None
    values = server_name.split('.')
    # cannot calculated base dn should be server.domain.tld
    # remove 'server' in dn
    if len(values) < 3:
        return None
    domain = ['ou=' + domain for domain in values[1:-2]]
    domain.append(f'o={values[-2]},o={values[-1]}')
    return ','.join(domain)
